#include "gpio.h"
#include "ram.h"

#define LED1_GPIO_CLK             RCC_AHB1Periph_GPIOE
#define LED1_PIN                  GPIO_Pin_2                
#define LED1_GPIO_PORT            GPIOE                       
#define LED1_SOURCE               GPIO_PinSource2

#define LED2_GPIO_CLK             RCC_AHB1Periph_GPIOE
#define LED2_PIN                  GPIO_Pin_3                
#define LED2_GPIO_PORT            GPIOE                       
#define LED2_SOURCE               GPIO_PinSource3

#define LED3_GPIO_CLK             RCC_AHB1Periph_GPIOE
#define LED3_PIN                  GPIO_Pin_4            
#define LED3_GPIO_PORT            GPIOE                       
#define LED3_SOURCE               GPIO_PinSource4

#define LED4_GPIO_CLK             RCC_AHB1Periph_GPIOE
#define LED4_PIN                  GPIO_Pin_5            
#define LED4_GPIO_PORT            GPIOE                       
#define LED4_SOURCE               GPIO_PinSource5

#define PUSH_BUTTON1_GPIO_CLK     RCC_AHB1Periph_GPIOE
#define PUSH_BUTTON1_PIN          GPIO_Pin_6                
#define PUSH_BUTTON1_GPIO_PORT    GPIOE                       
#define PUSH_BUTTON1_SOURCE       GPIO_PinSource6
#define PUSH_BUTTON1_EXTI_PORT    EXTI_PortSourceGPIOE
#define PUSH_BUTTON1_EXTI_PIN     EXTI_PinSource6
#define PUSH_BUTTON1_EXTI_LINE    EXTI_Line6

#define PUSH_BUTTON2_GPIO_CLK     RCC_AHB1Periph_GPIOC
#define PUSH_BUTTON2_PIN          GPIO_Pin_13                
#define PUSH_BUTTON2_GPIO_PORT    GPIOC                       
#define PUSH_BUTTON2_SOURCE       GPIO_PinSource13
#define PUSH_BUTTON2_EXTI_PORT    EXTI_PortSourceGPIOC
#define PUSH_BUTTON2_EXTI_PIN     EXTI_PinSource13
#define PUSH_BUTTON2_EXTI_LINE    EXTI_Line13

#define PUSH_BUTTON3_GPIO_CLK     RCC_AHB1Periph_GPIOC
#define PUSH_BUTTON3_PIN          GPIO_Pin_14                
#define PUSH_BUTTON3_GPIO_PORT    GPIOC                       
#define PUSH_BUTTON3_SOURCE       GPIO_PinSource14
#define PUSH_BUTTON3_EXTI_PORT    EXTI_PortSourceGPIOC
#define PUSH_BUTTON3_EXTI_PIN     EXTI_PinSource14
#define PUSH_BUTTON3_EXTI_LINE    EXTI_Line14

#define PUSH_BUTTON4_GPIO_CLK     RCC_AHB1Periph_GPIOC
#define PUSH_BUTTON4_PIN          GPIO_Pin_15
#define PUSH_BUTTON4_GPIO_PORT    GPIOC                       
#define PUSH_BUTTON4_SOURCE       GPIO_PinSource15
#define PUSH_BUTTON4_EXTI_PORT    EXTI_PortSourceGPIOC
#define PUSH_BUTTON4_EXTI_PIN     EXTI_PinSource15
#define PUSH_BUTTON4_EXTI_LINE    EXTI_Line15

#define GPO_TIMER                 TIM2
#define GPO_TIMER_CLK             RCC_APB1Periph_TIM2
#define GPO_TIMER_IRQn            TIM2_IRQn
#define GPO_TIMER_IRQHandler      TIM2_IRQHandler

#define GPI_EXTI1_IRQn            EXTI9_5_IRQn
#define GPI_EXTI1_IRQHandler      EXTI9_5_IRQHandler
#define GPI_EXTI2_IRQn            EXTI15_10_IRQn
#define GPI_EXTI2_IRQHandler      EXTI15_10_IRQHandler

// private variables
// LED blink periods
__IO uint16_t north_led_period;
__IO uint16_t south_led_period;
__IO uint16_t east_led_period;
__IO uint16_t west_led_period;
// Pushbuttons callbacks
void (*north_pb_callback)(void);
void (*south_pb_callback)(void);
void (*east_pb_callback)(void);
void (*west_pb_callback)(void);

// IRQ handler functions
void GPI_EXTI1_IRQHandler(void)
{
  if(EXTI_GetITStatus(PUSH_BUTTON1_EXTI_LINE) != RESET)
  {
    if(north_pb_callback!=0)  
      north_pb_callback();
    ram_set_bit(BIOLOID_PUSHBUTTON,0);
    /* Clear the EXTI line 0 pending bit */
    EXTI_ClearITPendingBit(PUSH_BUTTON1_EXTI_LINE);
  }
}

void GPI_EXTI2_IRQHandler(void)
{
  if(EXTI_GetITStatus(PUSH_BUTTON2_EXTI_LINE) != RESET)
  {
    if(south_pb_callback!=0)  
      south_pb_callback();
    ram_set_bit(BIOLOID_PUSHBUTTON,1);
    /* Clear the EXTI line 0 pending bit */
    EXTI_ClearITPendingBit(PUSH_BUTTON2_EXTI_LINE);
  }
  if(EXTI_GetITStatus(PUSH_BUTTON3_EXTI_LINE) != RESET)
  {
    if(east_pb_callback!=0)  
      east_pb_callback();
    ram_set_bit(BIOLOID_PUSHBUTTON,2);
    /* Clear the EXTI line 0 pending bit */
    EXTI_ClearITPendingBit(PUSH_BUTTON3_EXTI_LINE);
  }
  if(EXTI_GetITStatus(PUSH_BUTTON4_EXTI_LINE) != RESET)
  {
    if(west_pb_callback!=0)  
      west_pb_callback();
    ram_set_bit(BIOLOID_PUSHBUTTON,3);
    /* Clear the EXTI line 0 pending bit */
    EXTI_ClearITPendingBit(PUSH_BUTTON4_EXTI_LINE);
  }
}

void GPO_TIMER_IRQHandler(void)
{
  uint16_t capture;

  if(TIM_GetITStatus(GPO_TIMER, TIM_IT_CC1)!=RESET)
  {
    TIM_ClearITPendingBit(GPO_TIMER,TIM_IT_CC1);
    GPIO_ToggleBits(LED1_GPIO_PORT,LED1_PIN);
    capture = TIM_GetCapture1(GPO_TIMER);
    TIM_SetCompare1(GPO_TIMER, capture + north_led_period);
  }
  if(TIM_GetITStatus(GPO_TIMER, TIM_IT_CC2)!=RESET)
  {
    TIM_ClearITPendingBit(GPO_TIMER,TIM_IT_CC2);
    GPIO_ToggleBits(LED2_GPIO_PORT,LED2_PIN);
    capture = TIM_GetCapture2(GPO_TIMER);
    TIM_SetCompare2(GPO_TIMER, capture + south_led_period);
  }
  if(TIM_GetITStatus(GPO_TIMER, TIM_IT_CC3)!=RESET)
  {
    TIM_ClearITPendingBit(GPO_TIMER,TIM_IT_CC3);
    GPIO_ToggleBits(LED3_GPIO_PORT,LED3_PIN);
    capture = TIM_GetCapture3(GPO_TIMER);
    TIM_SetCompare3(GPO_TIMER, capture + east_led_period);
  }
  if(TIM_GetITStatus(GPO_TIMER, TIM_IT_CC4)!=RESET)
  {
    TIM_ClearITPendingBit(GPO_TIMER,TIM_IT_CC4);
    GPIO_ToggleBits(LED4_GPIO_PORT,LED4_PIN);
    capture = TIM_GetCapture4(GPO_TIMER);
    TIM_SetCompare4(GPO_TIMER, capture + west_led_period);
  }
}

// private functions

// public functions
void gpio_init(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  NVIC_InitTypeDef NVIC_InitStructure;
  EXTI_InitTypeDef EXTI_InitStructure;

  /* enable clocks */
  RCC_AHB1PeriphClockCmd(LED1_GPIO_CLK | LED2_GPIO_CLK | LED3_GPIO_CLK | LED4_GPIO_CLK, ENABLE);
  RCC_AHB1PeriphClockCmd(PUSH_BUTTON1_GPIO_CLK | PUSH_BUTTON2_GPIO_CLK | PUSH_BUTTON3_GPIO_CLK | PUSH_BUTTON4_GPIO_CLK, ENABLE);
  RCC_APB1PeriphClockCmd(GPO_TIMER_CLK,ENABLE);
  RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  /* GPIO Configuration */
  GPIO_InitStructure.GPIO_Pin = LED1_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  GPIO_Init(LED1_GPIO_PORT, &GPIO_InitStructure);
     
  GPIO_InitStructure.GPIO_Pin = LED2_PIN;
  GPIO_Init(LED2_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = LED3_PIN;
  GPIO_Init(LED3_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = LED4_PIN;
  GPIO_Init(LED4_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = PUSH_BUTTON1_PIN;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  GPIO_Init(PUSH_BUTTON1_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = PUSH_BUTTON2_PIN;
  GPIO_Init(PUSH_BUTTON2_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = PUSH_BUTTON3_PIN;
  GPIO_Init(PUSH_BUTTON3_GPIO_PORT, &GPIO_InitStructure);

  GPIO_InitStructure.GPIO_Pin = PUSH_BUTTON4_PIN;
  GPIO_Init(PUSH_BUTTON4_GPIO_PORT, &GPIO_InitStructure);

  // initialize private variables
  north_led_period=0;
  south_led_period=0;
  east_led_period=0;
  west_led_period=0;
  north_pb_callback=0;
  south_pb_callback=0;
  east_pb_callback=0;
  west_pb_callback=0;

  // initialize the timer interrupts
  NVIC_InitStructure.NVIC_IRQChannel = GPO_TIMER_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  /* LED's timer configuration */
  TIM_TimeBaseStructure.TIM_Period = 0xFFFF;
  TIM_TimeBaseStructure.TIM_Prescaler = 0;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit(GPO_TIMER,&TIM_TimeBaseStructure);
  TIM_Cmd(GPO_TIMER, ENABLE);
  TIM_PrescalerConfig(GPO_TIMER, 42000, TIM_PSCReloadMode_Immediate);
  TIM_SetClockDivision(GPO_TIMER,TIM_CKD_DIV2);

  /* Connect external interrupts */
  SYSCFG_EXTILineConfig(PUSH_BUTTON1_EXTI_PORT,PUSH_BUTTON1_EXTI_PIN);
  SYSCFG_EXTILineConfig(PUSH_BUTTON2_EXTI_PORT,PUSH_BUTTON2_EXTI_PIN);
  SYSCFG_EXTILineConfig(PUSH_BUTTON3_EXTI_PORT,PUSH_BUTTON3_EXTI_PIN);
  SYSCFG_EXTILineConfig(PUSH_BUTTON4_EXTI_PORT,PUSH_BUTTON4_EXTI_PIN);

  /* configure external interrupts */
  EXTI_InitStructure.EXTI_Line = PUSH_BUTTON1_EXTI_LINE | PUSH_BUTTON2_EXTI_LINE | PUSH_BUTTON3_EXTI_LINE | PUSH_BUTTON4_EXTI_LINE;
  EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
  EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Falling;
  EXTI_InitStructure.EXTI_LineCmd = ENABLE;
  EXTI_Init(&EXTI_InitStructure);

  NVIC_InitStructure.NVIC_IRQChannel = GPI_EXTI1_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

  NVIC_InitStructure.NVIC_IRQChannel = GPI_EXTI2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);
}

void gpio_set_led(led_t led_id)
{
  switch(led_id)
  {
    case NORTH_LED:
      GPIO_SetBits(LED1_GPIO_PORT,LED1_PIN);
      break;
    case SOUTH_LED:
      GPIO_SetBits(LED2_GPIO_PORT,LED2_PIN);
      break;
    case EAST_LED:
      GPIO_SetBits(LED3_GPIO_PORT,LED3_PIN);
      break;
    case WEST_LED:
      GPIO_SetBits(LED4_GPIO_PORT,LED4_PIN);
      break;
  }
}

void gpio_clear_led(led_t led_id)
{
  switch(led_id)
  {
    case NORTH_LED:
      GPIO_ResetBits(LED1_GPIO_PORT,LED1_PIN);
      break;
    case SOUTH_LED:
      GPIO_ResetBits(LED2_GPIO_PORT,LED2_PIN);
      break;
    case EAST_LED:
      GPIO_ResetBits(LED3_GPIO_PORT,LED3_PIN);
      break;
    case WEST_LED:
      GPIO_ResetBits(LED4_GPIO_PORT,LED4_PIN);
      break;
  }
}

void gpio_toggle_led(led_t led_id)
{
  switch(led_id)
  {
    case NORTH_LED:
      GPIO_ToggleBits(LED1_GPIO_PORT,LED1_PIN);
      break;
    case SOUTH_LED:
      GPIO_ToggleBits(LED2_GPIO_PORT,LED2_PIN);
      break;
    case EAST_LED:
      GPIO_ToggleBits(LED3_GPIO_PORT,LED3_PIN);
      break;
    case WEST_LED:
      GPIO_ToggleBits(LED4_GPIO_PORT,LED4_PIN);
      break;
  }
}

void gpio_blink_led(led_t led_id, int16_t period_ms)
{
  TIM_OCInitTypeDef  TIM_OCInitStructure;
  uint16_t capture;

  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;
  TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
  switch(led_id)
  {
    case NORTH_LED:
      if(period_ms>1)
      {
        north_led_period=period_ms;
        TIM_OCInitStructure.TIM_Pulse = north_led_period;
        TIM_OC1Init(GPO_TIMER, &TIM_OCInitStructure);
        TIM_OC1PreloadConfig(GPO_TIMER, TIM_OCPreload_Disable);
        capture = TIM_GetCounter(GPO_TIMER);
        TIM_SetCompare1(GPO_TIMER, capture + north_led_period);
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC1, ENABLE);
      }
      else
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC1, DISABLE);
      break;
    case SOUTH_LED:
      if(period_ms>1)
      {
        south_led_period=period_ms;
        TIM_OCInitStructure.TIM_Pulse = south_led_period;
        TIM_OC2Init(GPO_TIMER, &TIM_OCInitStructure);
        TIM_OC2PreloadConfig(GPO_TIMER, TIM_OCPreload_Disable);
        capture = TIM_GetCounter(GPO_TIMER);
        TIM_SetCompare2(GPO_TIMER, capture + south_led_period);
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC2, ENABLE);
      }
      else
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC2, DISABLE);
      break;
    case EAST_LED:
      if(period_ms>1)
      {
        east_led_period=period_ms;
        TIM_OCInitStructure.TIM_Pulse = east_led_period;
        TIM_OC3Init(GPO_TIMER, &TIM_OCInitStructure);
        TIM_OC3PreloadConfig(GPO_TIMER, TIM_OCPreload_Disable);
        capture = TIM_GetCounter(GPO_TIMER);
        TIM_SetCompare3(GPO_TIMER, capture + east_led_period);
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC3, ENABLE);
      }
      else
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC3, DISABLE);
      break;
    case WEST_LED:
      if(period_ms>1)
      {
        west_led_period=period_ms;
        TIM_OCInitStructure.TIM_Pulse = west_led_period;
        TIM_OC4Init(GPO_TIMER, &TIM_OCInitStructure);
        TIM_OC4PreloadConfig(GPO_TIMER, TIM_OCPreload_Disable);
        capture = TIM_GetCounter(GPO_TIMER);
        TIM_SetCompare4(GPO_TIMER, capture + west_led_period);
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC4, ENABLE);
      }
      else
        TIM_ITConfig(GPO_TIMER, TIM_IT_CC4, DISABLE);
      break;
  }
}

uint8_t gpio_is_pushbutton_pressed(pushbutton_t pb_id)
{
  switch(pb_id)
  {
    case NORTH_PB:
      if(GPIO_ReadInputDataBit(PUSH_BUTTON1_GPIO_PORT,PUSH_BUTTON1_PIN)==Bit_SET)
        return 0x01;
      else
        return 0x00;
      break;
    case SOUTH_PB:
      if(GPIO_ReadInputDataBit(PUSH_BUTTON2_GPIO_PORT,PUSH_BUTTON2_PIN)==Bit_SET)
        return 0x01;
      else
        return 0x00;
      break;
    case EAST_PB:
      if(GPIO_ReadInputDataBit(PUSH_BUTTON3_GPIO_PORT,PUSH_BUTTON3_PIN)==Bit_SET)
        return 0x01;
      else
        return 0x00;
      break;
    case WEST_PB:
      if(GPIO_ReadInputDataBit(PUSH_BUTTON4_GPIO_PORT,PUSH_BUTTON4_PIN)==Bit_SET)
        return 0x01;
      else
        return 0x00;
      break;
  }

  return 0x00;
}

void gpio_set_pushbutton_callback(pushbutton_t pb_id,void (*callback)(void))
{
  switch(pb_id)
  {
    case NORTH_PB:
      north_pb_callback=callback;
      break;
    case SOUTH_PB:
      south_pb_callback=callback;
      break;
    case EAST_PB:
      east_pb_callback=callback;
      break;
    case WEST_PB:
      west_pb_callback=callback;
      break;
  }
}
